﻿// Sjednocení API napříč prohlížeči
window.indexedDB = window.indexedDB || window["mozIndexedDB"] || window["webkitIndexedDB"] || window["msIndexedDB"];

window["IDBTransaction"] = window["IDBTransaction"] || window["webkitIDBTransaction"] || window["msIDBTransaction"];

window["IDBKeyRange"] = window["IDBKeyRange"] || window["webkitIDBKeyRange"] || window["msIDBKeyRange"];


// Pokud máte podtržený 'IDBKeyRange' (typescript hlásí chybu, tak odkomentujte):
// declare var IDBKeyRange;



(function () {
    var students = [
        {
            studentId: "jaw254",
            name: "Jan Janoušek",
            email: "jan.janousek@vsb.cz"
        },
        {
            studentId: "abc123",
            name: "Pepa Novák",
            email: "s1@sdfgsdg.1xyz"
        },
        {
            studentId: "cde456",
            name: "Jarda Novák",
            email: "s2@asdasfsdg.1xyz"
        },
        {
            studentId: "edc812",
            name: "Karel Gott",
            email: "s3@asdasd.1xyz"
        },
        {
            studentId: "ijk789",
            name: "Lucie Tichá",
            email: "s4@asdasd.1xyz"
        },
        {
            studentId: "poi123",
            name: "Adriana Chamrádová",
            email: "s5@sdsd.1xyz"
        }
    ];



    // Pomocné proměnné
    var DB_NAME = 'myFirstDb';
    var STUDENT_STORE_NAME = "students";
    var EMAIL_INDEX_NAME = "emailIndex";

    // Práce s databází
    var dbVersion = 1;

    // Otevřeme DB
    var idbRequest = window.indexedDB.open(DB_NAME, dbVersion);

    // Událost 'onupgradeneeded' pro vytváření/aktualizaci datastore
    idbRequest.addEventListener('upgradeneeded', function (e) {
        console.log("Stará verze DB:", e.oldVersion);

        var db: IDBDatabase = idbRequest.result;
        

        if (db.objectStoreNames.contains(STUDENT_STORE_NAME)) {
            db.deleteObjectStore(STUDENT_STORE_NAME);
        }

        // Vytvoříme nový datastore
        var studentsStore = db.createObjectStore(STUDENT_STORE_NAME, {
            keyPath: "studentId"
        });

        // Index na emailu
        studentsStore.createIndex(EMAIL_INDEX_NAME, "email", {
            unique: true
        });
    });



    // Databáze je připravena k práci
    idbRequest.addEventListener('success', function () {
        var db: IDBDatabase = idbRequest.result;
        
        var transaction = db.transaction([STUDENT_STORE_NAME], "readwrite");

        var store = transaction.objectStore(STUDENT_STORE_NAME);

        /*
        // Smazání položky
        deleteStudent(store, "jaw254", function(){
            console.log("Mazání bylo dokončeno");
        });
        
        
        // Vložení studentů
        insertStudents(store, function(){
            console.log("Studenti byli vloženi.");
        });
        
        
        // Načtení záznamu
        getStudent(store, "jaw254", function (student){
            console.log(student);
        
            // Aktualizace záznamu
            student.age = 25;
            updateStudent(store, student, function() {
                console.log("Aktualizace byla dokončena");
            });
        
        });
        
        
        findAllByCondition(store, function(students) {
            console.log("Vybraní studenti", students);
        });
        
        
        findByIndex(store, "jan.janousek@vsb.cz", function(student) {
            console.log(student);
        });
        
        findAllByRange(store, function (students) {
            console.log("Vybraní studenti", students);
        });
        */
    });


    // Vložení seznamu studentů
    function insertStudents(store: IDBObjectStore, callback) {
        var insertedCount = 0;
        for (var i = 0; i < students.length; i++) {
            var request = store.add(students[i]);

            request.addEventListener("success", function () {
                console.log("Záznam byl vložen", students[i]);

                insertedCount++;
                if (insertedCount == students.length) {
                    callback();
                }
            });

            request.addEventListener("error", function (e) {
                console.log("Došlo k chybě při vkládání záznamu", students[i]);
            });
        }
    }

    function deleteStudent(store: IDBObjectStore, key, callback) {
        var request = store.delete(key);

        request.addEventListener("success", function () {
            console.log("Záznam byl vymazán");
            callback();
        });

        request.addEventListener("error", function (e) {
            console.log("Došlo k chybě při mazání");
        });
    }


    function getStudent(store: IDBObjectStore, key, callback) {
        var request = store.get(key);

        request.addEventListener("success", function () {
            console.log("Záznam byl získán");
            callback(request.result);
        });

        request.addEventListener("error", function (e) {
            console.log("Došlo k chybě při získávání záznamu");
        });
    }



    function updateStudent(store: IDBObjectStore, student, callback) {
        var request = store.put(student);

        request.addEventListener("success", function () {
            console.log("Záznam byl aktualizován");
            callback();
        });

        request.addEventListener("error", function (e) {
            console.log("Došlo k chybě při aktualizaci záznamu");
        });
    }



    function findAllByCondition(store: IDBObjectStore, callback) {
        var request = store.openCursor();

        var students = [];

        request.addEventListener("success", function () {
            console.log("Kurzor byl otevřen");

            var cursor: IDBCursorWithValue = request.result;
            if (cursor)
            {
                if (cursor.value.studentId.indexOf("123") != -1) {
                    students.push(cursor.value);
                }
                cursor.continue();
            } else {
                callback(students);
            }
        });

        request.addEventListener("error", function (e) {
            console.log("Došlo k chybě při otevírání kurzoru");
        });
    }



    function findByIndex(store: IDBObjectStore, value, callback) {
        var emailIndex = store.index(EMAIL_INDEX_NAME);
        var request = emailIndex.get(value);

        request.addEventListener("success", function () {
            console.log("Záznam byl získán");

            // Pozor! Vrátí vždy jen 1 záznam (i když jich je více)
            callback(request.result);
        });

        request.addEventListener("error", function (e) {
            console.log("Došlo k chybě při získávání záznamu");
        });
    }


    function findAllByRange(store: IDBObjectStore, callback) {
        var range = IDBKeyRange.only("jaw254");
        /*
        range = IDBKeyRange.lowerBound("jaw254");
        range = IDBKeyRange.upperBound("jaw254", true); // Bez jaw254
        range = IDBKeyRange.bound("cde456", "jaw254");
        */
        var direction = "next";

        var request = store.openCursor(range, direction);

        var students = [];

        request.addEventListener("success", function () {
            console.log("Kurzor byl otevřen");

            var cursor: IDBCursorWithValue = request.result;
            if (cursor) {
                students.push(cursor.value);
                cursor.continue();
            } else {
                callback(students);
            }
        });


        request.addEventListener("error", function (e) {
            console.log("Došlo k chybě při otevírání kurzoru");
        });
    }
})();
